@@ -56,7 +56,8 @@ void cdf_create(const char *path, int cmode, int *ncidp)
   chunksizehint = 16777216; /* 16 MB */
 #endif
 
-  if ( cdiNcChunksizehint != CDI_UNDEFID ) chunksizehint = cdiNcChunksizehint;
+  if ( cdiNcChunksizehint != CDI_UNDEFID )
+    chunksizehint = (size_t)cdiNcChunksizehint;
 
   cdi_nc__create_funcp my_nc__create =
     (cdi_nc__create_funcp)namespaceSwitchGet(NSSWITCH_NC__CREATE).func;
@@ -101,7 +102,8 @@ int cdf_open(const char *path, int omode, int *ncidp)
       /*
       if ( chunksizehint < ChunkSizeMin ) chunksizehint = ChunkSizeMin;
       */
-      if ( cdiNcChunksizehint != CDI_UNDEFID ) chunksizehint = cdiNcChunksizehint;
+      if ( cdiNcChunksizehint != CDI_UNDEFID )
+        chunksizehint = (size_t)cdiNcChunksizehint;
 
       /* FIXME: parallel part missing */
       status = nc__open(path, omode, &chunksizehint, ncidp);
@@ -536,6 +538,19 @@ void  cdf_get_vara_text(int ncid, int varid, const size_t start[],
 }
 
 
+void  cdf_get_vara_uchar(int ncid, int varid, const size_t start[], const size_t count[], unsigned char *tp)
+{
+  int status;
+
+  status = nc_get_vara_uchar(ncid, varid, start, count, tp);
+
+  if ( CDF_Debug || status != NC_NOERR )
+    Message("ncid = %d varid = %d", ncid, varid);
+
+  if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
+}
+
+
 void cdf_put_var_double(int ncid, int varid, const double *dp)
 {
   int status;
@@ -723,7 +738,7 @@ void cdf_put_att_double(int ncid, int varid, const char *name, nc_type xtype,
 }
 
 
-void cdf_get_att_text(int ncid, int varid, char *name, char *tp)
+void cdf_get_att_text(int ncid, int varid, const char *name, char *tp)
 {
   int status;
 
@@ -735,8 +750,22 @@ void cdf_get_att_text(int ncid, int varid, char *name, char *tp)
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
 
+void cdf_get_att_string(int ncid, int varid, const char *name, char **tp)
+{
+#if  defined  (HAVE_NETCDF4)
+  int status;
 
-void cdf_get_att_int(int ncid, int varid, char *name, int *ip)
+  status = nc_get_att_string(ncid, varid, name, tp);
+
+  if ( CDF_Debug || status != NC_NOERR )
+    Message("ncid = %d varid = %d name = %s", ncid, varid, name);
+
+  if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
+#endif
+}
+
+
+void cdf_get_att_int(int ncid, int varid, const char *name, int *ip)
 {
   int status;
 
@@ -749,7 +778,7 @@ void cdf_get_att_int(int ncid, int varid, char *name, int *ip)
 }
 
 
-void cdf_get_att_double(int ncid, int varid, char *name, double *dp)
+void cdf_get_att_double(int ncid, int varid, const char *name, double *dp)
 {
   int status;
 
