@@ -1,4 +1,9 @@
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <inttypes.h>
+#include <sys/types.h>
 #include <stdlib.h>
 
 #include "cdi_cksum.h"
@@ -11,7 +16,7 @@ uint32_t cdiCheckSum(int type, int count, void *buffer)
   uint32_t s = 0U;
   xassert(count >= 0);
   size_t elemSize = (size_t)serializeGetSizeInCore(1, type, NULL);
-  memcrc_r_eswap(&s, (const unsigned char*) buffer, count, elemSize);
-  s = memcrc_finish(&s, elemSize * (size_t)count);
+  memcrc_r_eswap(&s, (const unsigned char *)buffer, (size_t)count, elemSize);
+  s = memcrc_finish(&s, (off_t)(elemSize * (size_t)count));
   return s;
 }
