@@ -13,13 +13,10 @@
 #include "error.h"
 #include "file.h"
 #include "binary.h"
+#include "stream_fcommon.h"
 #include "swap.h"
 
 
-#define SINGLE_PRECISION  4
-#define DOUBLE_PRECISION  8
-
-
 static int initIegLib      = 0;
 static int iegDefaultDprec = 0;
 
@@ -148,10 +145,10 @@ int iegCheckFiletype(int fileID, int *swap)
   size_t sblocklen = 0;
   size_t data = 0;
   size_t dimx = 0, dimy = 0;
-  int fact = 0, found = 0;
+  size_t fact = 0;
   unsigned char buffer[1048], *pbuf;
 
-  if ( fileRead(fileID, buffer, 4) != 4 ) return (found);
+  if ( fileRead(fileID, buffer, 4) != 4 ) return (0);
 
   blocklen  = get_UINT32(buffer);
   sblocklen = get_SUINT32(buffer);
@@ -163,7 +160,7 @@ int iegCheckFiletype(int fileID, int *swap)
     {
      *swap = 0;
       fact = 4;
-      if ( fileRead(fileID, buffer, blocklen+8) != blocklen+8 ) return (found);
+      if ( fileRead(fileID, buffer, blocklen+8) != blocklen+8 ) return (0);
       pbuf = buffer+(37+4)*4;    dimx = (size_t) get_UINT32(pbuf);
       pbuf = buffer+(37+5)*4;    dimy = (size_t) get_UINT32(pbuf);
       pbuf = buffer+blocklen+4;  data = (size_t) get_UINT32(pbuf);
@@ -172,7 +169,7 @@ int iegCheckFiletype(int fileID, int *swap)
     {
      *swap = 0;
       fact = 8;
-      if ( fileRead(fileID, buffer, blocklen+8) != blocklen+8 ) return (found);
+      if ( fileRead(fileID, buffer, blocklen+8) != blocklen+8 ) return (0);
       pbuf = buffer+(37+4)*4;    dimx = (size_t) get_UINT32(pbuf);
       pbuf = buffer+(37+5)*4;    dimy = (size_t) get_UINT32(pbuf);
       pbuf = buffer+blocklen+4;  data = (size_t) get_UINT32(pbuf);
@@ -181,7 +178,7 @@ int iegCheckFiletype(int fileID, int *swap)
     {
      *swap = 1;
       fact = 4;
-      if ( fileRead(fileID, buffer, sblocklen+8) != sblocklen+8 ) return (found);
+      if ( fileRead(fileID, buffer, sblocklen+8) != sblocklen+8 ) return (0);
       pbuf = buffer+(37+4)*4;     dimx = (size_t) get_SUINT32(pbuf);
       pbuf = buffer+(37+5)*4;     dimy = (size_t) get_SUINT32(pbuf);
       pbuf = buffer+sblocklen+4;  data = (size_t) get_SUINT32(pbuf);
@@ -190,7 +187,7 @@ int iegCheckFiletype(int fileID, int *swap)
     {
      *swap = 1;
       fact = 8;
-      if ( fileRead(fileID, buffer, sblocklen+8) != sblocklen+8 ) return (found);
+      if ( fileRead(fileID, buffer, sblocklen+8) != sblocklen+8 ) return (0);
       pbuf = buffer+(37+4)*4;     dimx = (size_t) get_SUINT32(pbuf);
       pbuf = buffer+(37+5)*4;     dimy = (size_t) get_SUINT32(pbuf);
       pbuf = buffer+sblocklen+4;  data = (size_t) get_SUINT32(pbuf);
@@ -198,8 +195,7 @@ int iegCheckFiletype(int fileID, int *swap)
 
   fileRewind(fileID);
 
-  if      ( data && dimx*dimy*fact == data ) found = 1;
-  else if ( data && dimx*dimy*8    == data ) found = 1;
+  int found = data && (dimx*dimy*fact == data || dimx*dimy*8 == data);
 
   if ( IEG_Debug )
     {
@@ -315,8 +311,8 @@ int iegDefData(iegrec_t *iegp, int prec, const void *data)
 
   iegp->dprec = dprec;
 
-  datasize = IEG_G_NumLon(iegp->igdb)*IEG_G_NumLat(iegp->igdb);
-  blocklen = datasize * dprec;
+  datasize = (size_t)IEG_G_NumLon(iegp->igdb) * (size_t)IEG_G_NumLat(iegp->igdb);
+  blocklen = datasize * (size_t)dprec;
 
   iegp->datasize = datasize;
 
@@ -386,7 +382,6 @@ int iegRead(int fileID, iegrec_t *iegp)
   char tmpbuffer[800], *tmpbuf = tmpbuffer;
   int dprec = 0;
   void *buffer;
-  int buffersize;
   int byteswap;
   int status;
 
@@ -478,16 +473,17 @@ int iegRead(int fileID, iegrec_t *iegp)
       return (-1);
     }
 
-  iegp->datasize = IEG_G_NumLon(iegp->igdb)*IEG_G_NumLat(iegp->igdb);
+  iegp->datasize = (size_t)IEG_G_NumLon(iegp->igdb)
+    * (size_t)IEG_G_NumLat(iegp->igdb);
 
   if ( IEG_Debug )
     Message("datasize = %lu", iegp->datasize);
 
   blocklen = binReadF77Block(fileID, byteswap);
 
-  buffersize = iegp->buffersize;
+  size_t buffersize = iegp->buffersize;
 
-  if ( buffersize < (int) blocklen )
+  if ( buffersize < blocklen )
     {
       buffersize = blocklen;
       buffer = iegp->buffer;
@@ -573,8 +569,8 @@ int iegWrite(int fileID, iegrec_t *iegp)
 
   binWriteF77Block(fileID, byteswap, blocklen);
 
-  datasize = iegp->igdb[4]*iegp->igdb[5];
-  blocklen = datasize * dprec;
+  datasize = (size_t)iegp->igdb[4] * (size_t)iegp->igdb[5];
+  blocklen = datasize * (size_t)dprec;
 
   binWriteF77Block(fileID, byteswap, blocklen);
 
