@@ -9,6 +9,25 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#if defined (HAVE_EXECINFO_H)
+#include <execinfo.h>
+#endif
+
+static
+void show_stackframe()
+{
+#if defined (HAVE_EXECINFO_H)
+  void *trace[16];
+  size_t i;
+  size_t trace_size = backtrace(trace, 16);
+  char **messages = backtrace_symbols(trace, trace_size);
+
+  fprintf(stderr, "[bt] Execution path:\n");
+  for ( i = 0; i < trace_size; ++i ) fprintf(stderr, "[bt] %s\n", messages[i]);
+  if ( messages ) free(messages);
+#endif
+}
+
 #include "dmemory.h"
 #include "resource_handle.h"
 #include "namespace.h"
@@ -130,7 +149,7 @@ reshListCreate(int namespaceID)
   LIST_LOCK();
   if (resHListSize <= namespaceID)
     {
-      resHList = (struct resHList_t*) xrealloc(resHList, (namespaceID + 1) * sizeof (resHList[0]));
+      resHList = (struct resHList_t *)xrealloc(resHList, (namespaceID + 1) * sizeof (resHList[0]));
       for (int i = resHListSize; i <= namespaceID; ++i)
         reshListClearEntry(i);
       resHListSize = namespaceID + 1;
@@ -154,10 +173,11 @@ reshListDestruct(int namespaceID)
       for ( int j = 0; j < resHList[namespaceID].size; j++ )
         {
           listElem_t *listElem = resHList[namespaceID].resources + j;
-          if (listElem->status != RESH_UNUSED)
+          if (listElem->status & RESH_IN_USE_BIT)
             listElem->res.v.ops->valDestroy(listElem->res.v.val);
         }
       free(resHList[namespaceID].resources);
+      resHList[namespaceID].resources = NULL;
       reshListClearEntry(namespaceID);
     }
   if (resHList[callerNamespaceID].resources)
@@ -172,6 +192,7 @@ static void listDestroy ( void )
   for (int i = resHListSize; i > 0; --i)
     if (resHList[i-1].resources)
       namespaceDelete(i-1);
+  resHListSize = 0;
   free(resHList);
   resHList = NULL;
   cdiReset();
@@ -209,26 +230,25 @@ void listSizeExtend()
 {
   int nsp = namespaceGetActive ();
   int oldSize = resHList[nsp].size;
-  int newListSize = oldSize + MIN_LIST_SIZE;
+  size_t newListSize = (size_t)oldSize + MIN_LIST_SIZE;
 
   resHList[nsp].resources = (listElem_t*) xrealloc(resHList[nsp].resources,
                                                    newListSize * sizeof(listElem_t));
 
   listElem_t *r = resHList[nsp].resources;
-  for (int i = oldSize; i < newListSize; ++i)
+  for (size_t i = (size_t)oldSize; i < newListSize; ++i)
     {
-      r[i].res.free.next = i + 1;
-      r[i].res.free.prev = i - 1;
+      r[i].res.free.next = (int)i + 1;
+      r[i].res.free.prev = (int)i - 1;
       r[i].status = RESH_UNUSED;
     }
 
   if (resHList[nsp].freeHead != -1)
-    r[resHList[nsp].freeHead].res.free.next
-      = newListSize - 1;
+    r[resHList[nsp].freeHead].res.free.prev = (int)newListSize - 1;
   r[newListSize-1].res.free.next = resHList[nsp].freeHead;
   r[oldSize].res.free.prev = -1;
   resHList[nsp].freeHead = oldSize;
-  resHList[nsp].size = newListSize;
+  resHList[nsp].size = (int)newListSize;
 }
 
 /**************************************************************/
@@ -237,13 +257,17 @@ static void
 reshPut_(int nsp, int entry, void *p, const resOps *ops)
 {
   listElem_t *newListElem = resHList[nsp].resources + entry;
-  int next = newListElem->res.free.next;
+  int next = newListElem->res.free.next,
+    prev = newListElem->res.free.prev;
   if (next != -1)
-    resHList[nsp].resources[next].res.free.prev = -1;
-  resHList[nsp].freeHead = next;
+    resHList[nsp].resources[next].res.free.prev = prev;
+  if (prev != -1)
+    resHList[nsp].resources[prev].res.free.next = next;
+  else
+    resHList[nsp].freeHead = next;
   newListElem->res.v.val = p;
   newListElem->res.v.ops = ops;
-  newListElem->status = RESH_ASSIGNED;
+  newListElem->status = RESH_DESYNC_IN_USE;
 }
 
 int reshPut ( void *p, const resOps *ops )
@@ -274,19 +298,18 @@ reshRemove_(int nsp, int idx)
   int curFree = resHList[nsp].freeHead;
   listElem_t *r = resHList[nsp].resources;
   r[idx].res.free.next = curFree;
+  r[idx].res.free.prev = -1;
   if (curFree != -1)
     r[curFree].res.free.prev = idx;
-  r[idx].status = RESH_UNUSED;
+  r[idx].status = RESH_DESYNC_DELETED;
   resHList[nsp].freeHead = idx;
 }
 
-void reshRemove ( cdiResH resH, const resOps * ops )
+void reshDestroy(cdiResH resH)
 {
   int nsp;
   namespaceTuple_t nspT;
 
-  LIST_INIT(1);
-
   LIST_LOCK();
 
   nsp = namespaceGetActive ();
@@ -296,7 +319,29 @@ void reshRemove ( cdiResH resH, const resOps * ops )
   xassert ( nspT.nsp == nsp
             && nspT.idx >= 0
             && nspT.idx < resHList[nsp].size
-            && resHList[nsp].resources[nspT.idx].status != RESH_UNUSED
+            && resHList[nsp].resources[nspT.idx].res.v.ops);
+
+  if (resHList[nsp].resources[nspT.idx].status & RESH_IN_USE_BIT)
+    reshRemove_(nsp, nspT.idx);
+
+  LIST_UNLOCK();
+}
+
+void reshRemove ( cdiResH resH, const resOps * ops )
+{
+  int nsp;
+  namespaceTuple_t nspT;
+
+  LIST_LOCK();
+
+  nsp = namespaceGetActive ();
+
+  nspT = namespaceResHDecode ( resH );
+
+  xassert ( nspT.nsp == nsp
+            && nspT.idx >= 0
+            && nspT.idx < resHList[nsp].size
+            && (resHList[nsp].resources[nspT.idx].status & RESH_IN_USE_BIT)
             && resHList[nsp].resources[nspT.idx].res.v.ops
             && resHList[nsp].resources[nspT.idx].res.v.ops == ops );
 
@@ -317,7 +362,7 @@ void reshReplace(cdiResH resH, void *p, const resOps *ops)
   while (resHList[nsp].size <= nspT.idx)
     listSizeExtend();
   listElem_t *q = resHList[nsp].resources + nspT.idx;
-  if (q->status != RESH_UNUSED)
+  if (q->status & RESH_IN_USE_BIT)
     {
       q->res.v.ops->valDestroy(q->res.v.val);
       reshRemove_(nsp, nspT.idx);
@@ -353,13 +398,17 @@ reshGetElem(const char *caller, cdiResH resH, const resOps *ops)
   else
     {
       LIST_UNLOCK();
+      show_stackframe();
       xabortC(caller, "Invalid namespace %d or index %d for resource handle %d when using namespace %d of size %d!",
               nspT.nsp, nspT.idx, (int)resH, nsp, resHList[nsp].size);
     }
 
   if ( !(listElem && listElem->res.v.ops == ops) )
-    xabortC(caller, "Invalid resource handle %d, list element not found!",
-            (int)resH);
+    {
+      show_stackframe();
+      xabortC(caller, "Invalid resource handle %d, list element not found!", (int)resH);
+    }
+
   return listElem;
 }
 
@@ -384,7 +433,7 @@ void reshGetResHListOfType ( int c, int * resHs, const resOps * ops )
   nsp = namespaceGetActive ();
 
   for ( i = 0; i < resHList[nsp].size && j < c; i++ )
-    if (resHList[nsp].resources[i].status != RESH_UNUSED
+    if ((resHList[nsp].resources[i].status & RESH_IN_USE_BIT)
         && resHList[nsp].resources[i].res.v.ops == ops)
       resHs[j++] = namespaceIdxEncode2(nsp, i);
 
@@ -404,7 +453,7 @@ cdiResHApply(enum cdiApplyRet (*func)(int id, void *res, const resOps *p,
   int nsp = namespaceGetActive ();
   enum cdiApplyRet ret = CDI_APPLY_GO_ON;
   for (int i = 0; i < resHList[nsp].size && ret > 0; ++i)
-    if (resHList[nsp].resources[i].status != RESH_UNUSED)
+    if (resHList[nsp].resources[i].status & RESH_IN_USE_BIT)
       ret = func(namespaceIdxEncode2(nsp, i),
                  resHList[nsp].resources[i].res.v.val,
                  resHList[nsp].resources[i].res.v.ops, data);
@@ -428,7 +477,7 @@ cdiResHFilterApply(const resOps *p,
   enum cdiApplyRet ret = CDI_APPLY_GO_ON;
   listElem_t *r = resHList[nsp].resources;
   for (int i = 0; i < resHList[nsp].size && ret > 0; ++i)
-    if (r[i].status != RESH_UNUSED && r[i].res.v.ops == p)
+    if ((r[i].status & RESH_IN_USE_BIT) && r[i].res.v.ops == p)
       ret = func(namespaceIdxEncode2(nsp, i), r[i].res.v.val,
                  data);
   LIST_UNLOCK();
@@ -454,7 +503,7 @@ int reshCountType ( const resOps * ops )
 
   listElem_t *r = resHList[nsp].resources;
   for ( i = 0; i < resHList[nsp].size; i++ )
-    countType += (r[i].status != RESH_UNUSED && r[i].res.v.ops == ops);
+    countType += ((r[i].status & RESH_IN_USE_BIT) && r[i].res.v.ops == ops);
 
   LIST_UNLOCK();
 
@@ -481,25 +530,30 @@ reshPackResource(int resH, const resOps *ops,
 
 static int getPackBufferSize(void *context)
 {
-  int nsp, i;
   int intpacksize, packBufferSize = 0;
 
-  nsp = namespaceGetActive ();
+  int nsp = namespaceGetActive ();
 
   /* pack start marker, namespace and sererator marker */
   packBufferSize += 3 * (intpacksize = serializeGetSize(1, DATATYPE_INT, context));
 
   /* pack resources, type marker and seperator marker */
   listElem_t *r = resHList[nsp].resources;
-  for ( i = 0; i < resHList[nsp].size; i++)
-    if (r[i].status == RESH_ASSIGNED)
+  for ( int i = 0; i < resHList[nsp].size; i++)
+    if (r[i].status & RESH_SYNC_BIT)
       {
-        xassert ( r[i].res.v.ops );
-
-        /* message plus frame of 2 ints */
-        packBufferSize +=
-          r[i].res.v.ops->valGetPackSize(r[i].res.v.val, context)
-          + 2 * intpacksize;
+        if (r[i].status == RESH_DESYNC_DELETED)
+          {
+            packBufferSize += 3 * intpacksize;
+          }
+        else if (r[i].status == RESH_DESYNC_IN_USE)
+          {
+            xassert ( r[i].res.v.ops );
+            /* message plus frame of 2 ints */
+            packBufferSize +=
+              r[i].res.v.ops->valGetPackSize(r[i].res.v.val, context)
+              + 2 * intpacksize;
+          }
       }
   /* end marker */
   packBufferSize += intpacksize;
@@ -528,7 +582,7 @@ void reshPackBufferCreate(char **packBuffer, int *packBufferSize, void *context)
   int nsp = namespaceGetActive ();
 
   int pBSize = *packBufferSize = getPackBufferSize(context);
-  char *pB = *packBuffer = (char*) xcalloc(1, *packBufferSize);
+  char *pB = *packBuffer = (char *)xcalloc(1, (size_t)pBSize);
 
   {
     int header[3] = { start, nsp, sep };
@@ -537,24 +591,29 @@ void reshPackBufferCreate(char **packBuffer, int *packBufferSize, void *context)
 
   listElem_t *r = resHList[nsp].resources;
   for ( i = 0; i < resHList[nsp].size; i++ )
-    if ( r[i].status == RESH_ASSIGNED)
+    if (r[i].status & RESH_SYNC_BIT)
       {
-        listElem_t * curr = r + i;
-        xassert ( curr->res.v.ops );
-
-        type = curr->res.v.ops->valTxCode ();
-
-        if ( ! type ) continue;
-
-        serializePack( &type, 1, DATATYPE_INT, * packBuffer,
-                       * packBufferSize, &packBufferPos, context);
-
-        curr->res.v.ops->valPack(curr->res.v.val,
-                                 pB, pBSize, &packBufferPos, context);
-
-        serializePack(&sep, 1, DATATYPE_INT, pB, pBSize, &packBufferPos, context);
-
-        curr->status = RESH_CLOSED;
+        if (r[i].status == RESH_DESYNC_DELETED)
+          {
+            enum { del_ints = 3 };
+            int temp[del_ints] = { RESH_DELETE, namespaceIdxEncode2(nsp, i), SEPARATOR };
+            serializePack(temp, del_ints, DATATYPE_INT,
+                          pB, pBSize, &packBufferPos, context);
+          }
+        else
+          {
+            listElem_t * curr = r + i;
+            xassert ( curr->res.v.ops );
+            type = curr->res.v.ops->valTxCode ();
+            if ( ! type ) continue;
+            serializePack(&type, 1, DATATYPE_INT, pB,
+                          pBSize, &packBufferPos, context);
+            curr->res.v.ops->valPack(curr->res.v.val,
+                                     pB, pBSize, &packBufferPos, context);
+            serializePack(&sep, 1, DATATYPE_INT,
+                          pB, pBSize, &packBufferPos, context);
+          }
+        r[i].status &= ~RESH_SYNC_BIT;
       }
 
   LIST_UNLOCK();
@@ -572,7 +631,7 @@ void reshSetStatus ( cdiResH resH, const resOps * ops, int status )
   namespaceTuple_t nspT;
   listElem_t * listElem;
 
-  xassert(ops && status != RESH_UNUSED);
+  xassert(ops && (status & RESH_IN_USE_BIT));
 
   LIST_INIT(1);
 
@@ -659,8 +718,8 @@ int reshListCompare ( int nsp0, int nsp1 )
     *resources1 = resHList[nsp1].resources;
   for (i = 0; i < listSizeMin; i++)
     {
-      int occupied0 = resources0[i].status != RESH_UNUSED,
-        occupied1 = resources1[i].status != RESH_UNUSED;
+      int occupied0 = (resources0[i].status & RESH_IN_USE_BIT) != 0,
+        occupied1 = (resources1[i].status & RESH_IN_USE_BIT) != 0;
       /* occupation mismatch ? */
       int diff = occupied0 ^ occupied1;
       valCompare |= (diff << cdiResHListOccupationMismatch);
@@ -682,11 +741,11 @@ int reshListCompare ( int nsp0, int nsp1 )
     }
   /* find resources in nsp 0 beyond end of nsp 1 */
   for (int j = listSizeMin; j < resHList[nsp0].size; ++j)
-    valCompare |= ((resources0[j].status != RESH_UNUSED)
+    valCompare |= (((resources0[j].status & RESH_IN_USE_BIT) != 0)
                    << cdiResHListOccupationMismatch);
   /* find resources in nsp 1 beyond end of nsp 0 */
   for (; i < resHList[nsp1].size; ++i)
-    valCompare |= ((resources1[i].status != RESH_UNUSED)
+    valCompare |= (((resources1[i].status & RESH_IN_USE_BIT) != 0)
                    << cdiResHListOccupationMismatch);
 
   LIST_UNLOCK();
@@ -725,7 +784,7 @@ void reshListPrint(FILE *fp)
       for ( j = 0; j < resHList[i].size; j++ )
         {
           curr = resHList[i].resources + j;
-          if (curr->status != RESH_UNUSED)
+          if (!(curr->status & RESH_IN_USE_BIT))
             {
               curr->res.v.ops->valPrint(curr->res.v.val, fp);
               fprintf ( fp, "\n" );
